/*
 * AutomatonFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "AutomatonFromStringLexer.h"
#include <exception/CommonException.h>
#include <alib/iterator>

namespace automaton {

AutomatonFromStringLexer::Token AutomatonFromStringLexer::next(std::istream& in) {
	AutomatonFromStringLexer::Token token;
	token.type = TokenType::ERROR;
	token.value = "";
	token.raw = "";
	char character;

L0:
	character = in.get();
	if ( in.eof ( ) || character == EOF ) { // for some reason the in stream was not setting eof bit for automata input without last newline
		token.type = TokenType::TEOF;
		return token;
	} else if(character == '\r' || character == '\n') {
		token.type = TokenType::NEW_LINE;
		token.value += character;
		token.raw += character;
		return token;
	} else if ( isspace ( character ) ) {
		token.raw += character;
		goto L0;
	} else if(character == '<') {
		token.type = TokenType::OUT;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '>') {
		token.type = TokenType::IN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '|') {
		token.type = TokenType::SEPARATOR;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '-') {
		token.type = TokenType::NONE;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '#') {
		token.value += character;
		token.raw += character;
		goto L1;
	} else if(in.clear(), in.unget(), in >> ext::string ( "MISNFA" ) ) {
		token.type = TokenType::MULTI_INITIAL_STATE_NFA;
		token.value = "MISNFA";
		token.raw += "MISNFA";
		return token;
	} else if(in.clear(), in >> ext::string ( "ENFA" ) ) {
		token.type = TokenType::EPSILON_NFA;
		token.value = "ENFA";
		token.raw += "ENFA";
		return token;
	} else if(in.clear(), in >> ext::string ( "NFA" ) ) {
		token.type = TokenType::NFA;
		token.value = "NFA";
		token.raw += "NFA";
		return token;
	} else if(in.clear(), in >> ext::string ( "DFA" ) ) {
		token.type = TokenType::DFA;
		token.value = "DFA";
		token.raw += "DFA";
		return token;
	} else {
		in.clear();
		putback(in, token);
		token.raw = "";
		token.type = TokenType::ERROR;
		return token;
	}
L1:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == 'E') {
		token.type = TokenType::EPSILON;
		token.value += character;
		token.raw += character;
		return token;
	} else {
		in.clear ( );
		in.unget ( );
		putback(in, token);
		token.raw = "";
		token.type = TokenType::ERROR;
		return token;
	}
}

AutomatonFromStringLexer::Token AutomatonFromStringLexer::peek ( std::istream & in ) {
	Token token = next ( in );
	putback ( in, token );
	return token;
}

void AutomatonFromStringLexer::putback ( std::istream& in, const std::string & data ) {
	in.clear();

	for ( char character : ext::make_reverse ( data ) )
		in.putback ( character );
}

void AutomatonFromStringLexer::putback ( std::istream& in, const Token & token ) {
	putback ( in, token.raw );
}

bool AutomatonFromStringLexer::test ( std::istream & in, const std::string & value ) {
	if ( testAndConsume ( in, value ) ) {
		in.clear ( );
		putback ( in, value );
		return true;
	} else {
		return false;
	}
}

bool AutomatonFromStringLexer::testAndConsume ( std::istream & in, const std::string & value ) {
	return ( bool ) ( in >> ext::string ( value ) );
}

void AutomatonFromStringLexer::consume ( std::istream & in, const std::string & value ) {
	if ( ! testAndConsume ( in, value ) )
		throw exception::CommonException ( "Can't consume " + value + " from input stream." );
}

} /* namespace automaton */
